// Code generated by counterfeiter. DO NOT EDIT.
package osshim

import (
	"os"
	"time"
)

type Os interface {
	FindProcess(pid int) (*os.Process, error)
	Hostname() (name string, err error)
	Expand(s string, mapping func(string) string) string
	Clearenv()
	Environ() []string
	Chtimes(name string, atime time.Time, mtime time.Time) error
	MkdirAll(path string, perm os.FileMode) error
	Exit(code int)
	Fictional(lol ...string)
}
