package extra // import "github.com/maxbrunsfeld/counterfeiter/v6/fixtures/type_aliases/extra"

import "github.com/maxbrunsfeld/counterfeiter/v6/fixtures/type_aliases/extra/primitive"

type M = primitive.M
