// Code generated by counterfeiter. DO NOT EDIT.
package packagemodeshim

import (
	"github.com/maxbrunsfeld/counterfeiter/v6/fixtures/packagemode"
)

//go:generate go run github.com/maxbrunsfeld/counterfeiter/v6 -generate
//counterfeiter:generate . Packagemode

// Packagemode is a generated interface representing the exported functions
// in the github.com/maxbrunsfeld/counterfeiter/v6/fixtures/packagemode package.
type Packagemode interface {
	Arg(arg1 int) string
	Args() []string
	Bool(arg1 string, arg2 bool, arg3 string) *bool
	BoolVar(arg1 *bool, arg2 string, arg3 bool, arg4 string)
}

type PackagemodeShim struct{}

func (p *PackagemodeShim) Arg(arg1 int) string {
	return packagemode.Arg(arg1)
}

func (p *PackagemodeShim) Args() []string {
	return packagemode.Args()
}

func (p *PackagemodeShim) Bool(arg1 string, arg2 bool, arg3 string) *bool {
	return packagemode.Bool(arg1, arg2, arg3)
}

func (p *PackagemodeShim) BoolVar(arg1 *bool, arg2 string, arg3 bool, arg4 string) {
	packagemode.BoolVar(arg1, arg2, arg3, arg4)
}

var _ Packagemode = new(PackagemodeShim)
